import java.io.*;
import java.util.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.io.*;

import comm2j.Comm2J;
import comm2j.Read2J;
import comm2j.Message;
import comm2j.ByteMessage;
import comm2j.DateMessage;
import comm2j.ShortMessage;
import comm2j.IntegerMessage;
import comm2j.LongMessage;
import comm2j.StringMessage;
import comm2j.VectorMessage;

public class TestComm extends MIDlet implements CommandListener
{
  private Display display;
  private Form main;
  private Form text;
  private Form prefs;
  private ChoiceGroup langChoose;
  private ChoiceGroup modeChoose;
  private Comm2J comm;
  private TextField hostTf;
  private TextField portTf;

  private String DEFAULTHOST  = "localhost";
  private String DEFAULTPORT  = "6789";
  private int DEFAULTMODE     = Read2J.COMPRESSED;

  public TestComm()
  {
    display         = Display.getDisplay(this);
  }

  protected void destroyApp(boolean unconditional)
  {
  }

  public void pauseApp()
  {
    display.setCurrent(null);
    main            = null;
    text            = null;
  }

  public void startApp()
  {
    Runtime runtime = Runtime.getRuntime();
    runtime.gc();

    main            = new Form("Comm2J Demo");
    text            = new Form("Text");
    prefs           = new Form("Preferences");

    main.append("Choose a language\n");
    text.append("...\n");

    modeChoose      = new ChoiceGroup("Mode", Choice.EXCLUSIVE);
    modeChoose.append("Compressed",null);
    modeChoose.append("Raw",null);
    if(DEFAULTMODE == Read2J.COMPRESSED)
      modeChoose.setSelectedIndex(0, true);
    else
      modeChoose.setSelectedIndex(1, true);

    langChoose      = new ChoiceGroup("Language", Choice.EXCLUSIVE);
    langChoose.append("Italiano",null);
    langChoose.append("Deutsch",null);
    langChoose.append("Al-'arabya",null);
    langChoose.append("English",null);

    hostTf          = new TextField("Host", DEFAULTHOST, 30, TextField.URL);
    portTf          = new TextField("Port", DEFAULTPORT, 5, TextField.NUMERIC);

    prefs.append("Server options:");
    prefs.append(langChoose);
    prefs.append(hostTf);
    prefs.append(portTf);
    prefs.append(modeChoose);

    prefs.addCommand(new Command("Cancel", Command.CANCEL, 1));
    prefs.addCommand(new Command("OK", Command.OK, 1));
    prefs.setCommandListener(this);

    text.addCommand(new Command("Exit", Command.EXIT, 1));
    text.addCommand(new Command("Prefs", Command.SCREEN, 1));
    main.addCommand(new Command("Exit", Command.EXIT, 1));
    main.addCommand(new Command("Prefs", Command.SCREEN, 1));
    main.setCommandListener(this);
    text.setCommandListener(this);

    try
    {
      Form intro    = new Form("Comm2J");
      intro.append(new ImageItem("",
                                  Image.createImage("/duke.png"),
                                  ImageItem.LAYOUT_CENTER,
                                  "Image Cannot be shown"));
      intro.append("Comm2J v. 1.0");
      intro.append("Demo Application");

      display.setCurrent(intro);
    }
    catch(Exception e)
    {
      System.err.println("Exception: [" + e.toString() + "]");
    }
    new Thread(new WaitForIntro(display, main, 2000)).start();
  }

  public void commandAction(Command c, Displayable s)
  {
    if (c.getCommandType() == Command.EXIT)
    {
      destroyApp(false);
      notifyDestroyed();
    }
    else if (c.getCommandType() == Command.SCREEN && (s == main || s == text))
    {
      prefs.setTitle("Preferences");
      display.setCurrent(prefs);
    }
    else if (c.getCommandType() == Command.CANCEL && (s == prefs))
    {
      display.setCurrent(main);
    }
    else if (c.getCommandType() == Command.OK && (s == prefs))
    {
      int indSel    = langChoose.getSelectedIndex();
      int mode;

      if(modeChoose.getSelectedIndex() == 0)
        mode        = Read2J.COMPRESSED;
      else
        mode        = Read2J.RAW;

      String str    = null;
      if(indSel == 0)
      {
        str         = getStringFromServer(hostTf.getString(), portTf.getString(), mode, "IT");
        if(str != null)
          text.set(0, new StringItem("", str));
      }
      if(indSel == 1)
      {
        str         = getStringFromServer(hostTf.getString(), portTf.getString(), mode, "DE");
        if(str != null)
          text.set(0, new StringItem("", str));
      }
      if(indSel == 2)
      {
        str         = getStringFromServer(hostTf.getString(), portTf.getString(), mode, "AR");
        if(str != null)
          text.set(0, new StringItem("", str));
      }
      if(indSel == 3)
      {
        str         = getStringFromServer(hostTf.getString(), portTf.getString(), mode, "EN");
        if(str != null)
          text.set(0, new StringItem("", str));
      }

      if(str != null && !str.equals(""))
        display.setCurrent(text);
    }
  }

  private String getStringFromServer(String host, String port, int mode, String lang)
  {
    Message msg, nMsg, outMsg;
    VectorMessage vMsg;
    Calendar cal;
    String ret      = "";

    cal             = Calendar.getInstance();

    prefs.setTitle("Connecting...");
    comm            = new Comm2J(this, display, host, port);
    comm.setMode(mode);
    comm.connect();
    if(comm.isConnected())
    {
      outMsg        = new StringMessage(lang);
      vMsg          = new VectorMessage();
      prefs.setTitle("Sending data...");
      if(comm.writeMessageToStream(outMsg) != -1)
        {
          prefs.setTitle("Receiving data...");
          vMsg      = (VectorMessage) comm.readMessageFromStream();
          prefs.setTitle("Decoding data...");
          for(Enumeration e = vMsg.elements(); e.hasMoreElements();)
          {
            msg     = (Message) e.nextElement();
            if(new ByteMessage().getName().equals(msg.getName()))
            {
              nMsg  = (ByteMessage) msg;
              ret      += " - b: [" + nMsg.toString() + "]";
            }
            else if(new DateMessage().getName().equals(msg.getName()))
            {
              nMsg  = (DateMessage) msg;
              cal.setTime((Date) nMsg.getMessage());
              ret  += " - d: [" + cal.get(Calendar.DAY_OF_MONTH) + "." + (cal.get(Calendar.MONTH) + 1) + "." + cal.get(Calendar.YEAR) + "-" +
                       cal.get(Calendar.HOUR_OF_DAY) + ":" + cal.get(Calendar.MINUTE) + "." + cal.get(Calendar.SECOND) + "]";
            }
            else if(new ShortMessage().getName().equals(msg.getName()))
            {
              nMsg  = (ShortMessage) msg;
              ret  += " - s: [" + nMsg.toString() + "]";
            }
            else if(new IntegerMessage().getName().equals(msg.getName()))
            {
              nMsg  = (IntegerMessage) msg;
              ret  += " - i: [" + nMsg.toString() + "]";
            }
            else if(new LongMessage().getName().equals(msg.getName()))
            {
              nMsg  = (LongMessage) msg;
              ret  += " - l: [" + nMsg.toString() + "]";
            }
            else if(new StringMessage().getName().equals(msg.getName()))
            {
              nMsg  = (StringMessage) msg;
              ret  += (String) nMsg.getMessage();
            }
          }
        }
      comm.disconnect();
    }

    return ret;
  }

  class WaitForIntro implements Runnable
  {
    Form main;
    Display display;
    int sleepTime;

    public WaitForIntro(Display display, Form main, int sleepTime)
    {
      this.display    = display;
      this.main       = main;
      this.sleepTime  = sleepTime;
    }

    public void run()
    {
      try
      {
        Thread.currentThread().yield();

        Thread.currentThread().sleep(sleepTime);
        display.setCurrent(main);
      }
      catch(Exception e)
      {
        System.err.println("Exception: [" + e.toString() + "]");
      }
    }
  }
}
