import java.io.*; 
import java.net.*; 
import java.util.*;

import comm2j.Message;
import comm2j.ByteMessage;
import comm2j.DateMessage;
import comm2j.IntegerMessage;
import comm2j.LongMessage;
import comm2j.ShortMessage;
import comm2j.StringMessage;
import comm2j.VectorMessage;
import comm2j.Read2J;

class TCPServer
{
  public static void main(String argv[]) throws Exception
  {
    int mode                  = Read2J.COMPRESSED;
    int port                  = 6789;

    if(argv.length > 0)
    {
      for(int l = 0; l < argv.length; l++)
      {
        if(argv[l].equals("--compressed") || argv[l].equals("-c"))
        {
          mode                = Read2J.COMPRESSED;
        }
        else if(argv[l].equals("--raw") || argv[l].equals("-r"))
        {
          mode                = Read2J.RAW;
        }
        else if(argv[l].equals("--port") || argv[l].equals("-p"))
        {
          port                = Integer.valueOf(argv[l + 1]).intValue();
          l++;
        }
        else if(argv[l].equals("--help") || argv[l].equals("-h"))
        {
          System.out.println("Usage: java TCPServer [opt]\n");
          System.out.println("Valid options are:");
          System.out.println("--compressed [-c]      Use compression to send data (default)");
          System.out.println("--raw [-r]             Don't use compression to send data");
          System.out.println("--port [-p]            Use port <port> to send/receive data (default: 6789)");
          System.exit(0);
        }
      }
    }

    ServerSocket serverSocket = new ServerSocket(port);
    if(mode == Read2J.COMPRESSED)
    {
      System.out.print("Using compression - ");
    }
    System.out.println("Listening on port " + port + "...");
    while(true)
      {
        Socket cSocket        = serverSocket.accept();
        InputStream is        = cSocket.getInputStream();
        OutputStream os       = cSocket.getOutputStream();
        String str            = null;
        Read2J read2j         = new Read2J();
        read2j.setMode(mode);

        Message inMsg         = new StringMessage();
        Message intMsg        = new IntegerMessage();
        Message shMsg         = new ShortMessage();
        Message lnMsg         = new LongMessage();
        Message bMsg          = new ByteMessage();
        Message dMsg          = new DateMessage();
        Message outMsg        = new StringMessage();
        VectorMessage vMsg    = new VectorMessage();

        inMsg                 = read2j.readMessageFromStream(is);
        if(inMsg.isMessageSet())
          str                 = (String) inMsg.getMessage();

        System.out.println("Received [" + str + "] from [" + cSocket.getInetAddress().getHostAddress() + "]");

        if("AR".equals(str))
        {
          intMsg.setMessage(new Integer(1));
          shMsg.setMessage(new Short((short) 11));
          lnMsg.setMessage(new Long((long) 111));
          bMsg.setMessage(new Byte((byte) 65));
          outMsg.setMessage("Ana aktabu jumlatun...");
        }
        if("EN".equals(str))
        {
          intMsg.setMessage(new Integer(2));
          shMsg.setMessage(new Short((short) 12));
          lnMsg.setMessage(new Long((long) 112));
          bMsg.setMessage(new Byte((byte) 66));
          outMsg.setMessage("I just write a little sentece...");
        }
        if("DE".equals(str))
        {
          intMsg.setMessage(new Integer(3));
          shMsg.setMessage(new Short((short) 13));
          lnMsg.setMessage(new Long((long) 113));
          bMsg.setMessage(new Byte((byte) 67));
          outMsg.setMessage("Ich schreibe eine Satz um zu demonstrieren dass das Programm luft...");
        }
        if("IT".equals(str))
        {
          intMsg.setMessage(new Integer(4));
          shMsg.setMessage(new Short((short) 14));
          lnMsg.setMessage(new Long((long) 114));
          bMsg.setMessage(new Byte((byte) 68));
          outMsg.setMessage("Scrivo qualche frase, giusto per dare al sistema la possibilita' di comprimere qualcosa!\nSe tutto va come deve andare, dovrei avere una quantita' di dati non superiore al normale...\nSe tutto va male, sono rovinato...\nIo speriamo che me la cavo!");
        }
        dMsg.setMessage(new Date());

        if(outMsg.isMessageSet())
        {
          vMsg.addMessage(outMsg);
          vMsg.addMessage(intMsg);
          vMsg.addMessage(shMsg);
          vMsg.addMessage(lnMsg);
          vMsg.addMessage(bMsg);
          vMsg.addMessage(dMsg);
          read2j.writeMessageToStream(vMsg, os);
        }

        is.close();
        os.close();
        cSocket.close();
      }
  }
}